@@ -1,6 +1,6 @@
-/* plot_phylo.c (2009-10-03) */
+/* plot_phylo.c (2011-06-23) */
 
-/* Copyright 2004-2009 Emmanuel Paradis
+/* Copyright 2004-2011 Emmanuel Paradis
 
 /* This file is part of the R-package `ape'. */
 /* See the file ../COPYING for licensing issues. */
@@ -37,12 +37,11 @@ void node_depth(int *ntip, int *nnode, int *edge1, int *edge2,
 void node_height(int *ntip, int *nnode, int *edge1, int *edge2,
 		int *nedge, double *yy)
 {
-    int i, k, n;
+    int i, n;
     double S;
 
     /* The coordinates of the tips have been already computed */
 
-    k = 1;
     S = 0;
     n = 0;
     for (i = 0; i < *nedge; i++) {
@@ -59,14 +58,13 @@ void node_height(int *ntip, int *nnode, int *edge1, int *edge2,
 void node_height_clado(int *ntip, int *nnode, int *edge1, int *edge2,
 		       int *nedge, double *xx, double *yy)
 {
-    int i, k, n;
+    int i, n;
     double S;
 
     node_depth(ntip, nnode, edge1, edge2, nedge, xx);
 
     /* The coordinates of the tips have been already computed */
 
-    k = 1;
     S = 0;
     n = 0;
     for (i = 0; i < *nedge; i++) {
